unit DabDepartment;

interface

uses
  Classes, DabEmployee, CustomVisitor;

type
  TDabDepartment = class(TCollectionItem)
  private
    FDepartmentName : string;
    FEmployees      : TDabEmployees;

    procedure SetEmployees(const Value : TDabEmployees);
  protected
  public
    constructor Create(Collection : TCollection); override;
    destructor  Destroy; override;

    procedure Assign(Source : TPersistent); override;
    // Needed for Visitor
    procedure Accept(Visitor : TCustomVisitor); virtual;
  published
    property DepartmentName : string read FDepartmentName write FDepartmentName;
    property Employees      : TDabEmployees read FEmployees write SetEmployees;
  end;


  TDabDepartments = class(TOwnedCollection)
  private
  protected
    function  GetItem(Index : Integer) : TDabDepartment;
    procedure SetItem(Index : Integer;Value : TDabDepartment);
  public
    function  Add : TDabDepartment;
    procedure AddNew(const ADepartmentName : string);
    // Needed for Visitor
    procedure Accept(Visitor : TCustomVisitor); virtual;
    function  FindItemID(ID : Integer) : TDabDepartment;
    function  Insert(Index : Integer) : TDabDepartment;

    property Items[Index : Integer] : TDabDepartment read GetItem
                                                     write SetItem; default;
  published
  end;


implementation

uses
  SysUtils;
  
{ TDabDepartment }

constructor TDabDepartment.Create(Collection : TCollection);
begin
  inherited;
  FEmployees := TDabEmployees.Create(Self,TDabEmployee);
end;


destructor TDabDepartment.Destroy;
begin
  FreeAndNil(FEmployees);
  inherited;
end;


procedure TDabDepartment.SetEmployees(const Value : TDabEmployees);
begin
  FEmployees.Assign(Value);
end;


procedure TDabDepartment.Assign(Source : TPersistent);
begin
  if Source is TDabDepartment then begin
    DepartmentName := TDabDepartment(Source).DepartmentName;
    Employees     := TDabDepartment(Source).Employees;
  end;
end;


procedure TDabDepartment.Accept(Visitor : TCustomVisitor);
begin
  Visitor.Visit(Self);
  Employees.Accept(Visitor);
end;


{ TDabDepartments }

function TDabDepartments.GetItem(Index : Integer) : TDabDepartment;
begin
  Result := TDabDepartment(inherited GetItem(Index));
end;


procedure TDabDepartments.SetItem(Index : Integer;Value : TDabDepartment);
begin
  inherited SetItem(Index,Value);
end;


function TDabDepartments.Add : TDabDepartment;
begin
  Result := TDabDepartment(inherited Add);
end;


procedure TDabDepartments.AddNew(const ADepartmentName : string);
begin
  Add.DepartmentName := ADepartmentName;
end;


procedure TDabDepartments.Accept(Visitor : TCustomVisitor);
  var
    i : Integer;
begin
  Visitor.Visit(Self);

  for i := 0 to Count - 1 do begin
    Items[i].Accept(Visitor);
  end;
end;


function TDabDepartments.FindItemID(ID : Integer) : TDabDepartment;
begin
  Result := TDabDepartment(inherited FindItemID(ID));
end;


function TDabDepartments.Insert(Index : Integer) : TDabDepartment;
begin
  Result := TDabDepartment(inherited Insert(Index));
end;


end.
 